/*
 * Copyright (c) 2021-2022, Luca Fulchir <luker@fenrirproject.org>
 *
 * This file is part of dfim.
 *
 * dfim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * dfim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with dfim.  If not, see <https://www.gnu.org/licenses/>.
 */

use super::device;
use super::errors;
use super::trgt;

#[derive(::serde::Deserialize, Clone)]
#[serde(deny_unknown_fields)]
pub struct Include {
    #[serde(default)]
    pub devices: Vec<device::Device>,
    #[serde(default)]
    pub targets: Vec<trgt::Target>,
}

pub fn parse_include(
    logger: &::slog::Logger,
    conf_path: &::std::path::Path,
) -> std::result::Result<Include, errors::ConfigError> {
    let conf_device = ::std::fs::File::open(conf_path)?;
    let mut extensions = ::ron::extensions::Extensions::default();
    extensions.insert(::ron::extensions::Extensions::IMPLICIT_SOME);
    extensions.insert(::ron::extensions::Extensions::UNWRAP_NEWTYPES);
    // TODO: wait for new RON release
    //match ::ron::Options::default().
    // with_default_extensions(extensions).from_reader(&conf_device) {
    let cfg: Include = match ::ron::de::from_reader(&conf_device) {
        Ok(cfg) => cfg,
        Err(repr) => {
            ::slog::error!(logger, "on file: {:?}", &conf_path);
            ::slog::error!(logger, "{}", repr);
            return Err(errors::ConfigError::Parsing(repr));
        }
    };
    return Ok(cfg);
}
