/*
 * Copyright (c) 2021-2022, Luca Fulchir <luker@fenrirproject.org>
 *
 * This file is part of dfim.
 *
 * dfim is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * dfim is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with dfim.  If not, see <https://www.gnu.org/licenses/>.
 */

use crate::config;
use crate::state::s_trgt::{
    CheckStatus, GetCurrentResult, NextTarget, TargetApply,
};

pub struct BTRFS(crate::config::trgt::fs::btrfs::BTRFS);

impl BTRFS {
    pub fn new(conf: &crate::config::trgt::fs::btrfs::BTRFS) -> Self {
        BTRFS(conf.clone())
    }
}

impl TargetApply for BTRFS {
    fn dev_to_config(
        &mut self,
        logger: &::slog::Logger,
        cfg: &crate::state::cfg::Cfg,
        cfg_dev: &crate::config::device::Device,
        dev: &::udev::Device,
    ) -> Result<crate::config::trgt::Target, ::std::io::Error> {
        // we don't have to adapt the config to the device like gpt does
        Ok(config::trgt::Target::BTRFS(self.0.clone()))
    }
    fn get_current(
        &mut self,
        logger: &::slog::Logger,
        cfg: &crate::state::cfg::Cfg,
        cfg_dev: &crate::config::device::Device,
        dev: &::udev::Device,
    ) -> Result<GetCurrentResult, ::std::io::Error> {
        Ok(GetCurrentResult {
            status: CheckStatus::DoNotApply,
            conf: crate::config::trgt::Target::Dummy,
            next: Vec::new(),
        })
    }

    fn apply(
        &mut self,
        logger: &::slog::Logger,
        cfg: &crate::state::cfg::Cfg,
        cfg_dev: &crate::config::device::Device,
        dev: &::udev::Device,
    ) -> Result<Vec<NextTarget>, ::std::io::Error> {
        Ok(Vec::new())
    }
}
